from __future__ import print_function


def my_i (args):
    print ("i", args)

def my_ii (args):
    print ("ii", args)

def my_iii (args):
    print ("iii", args)

def my_c (cls, args):
    print ("c", args)

def my_cc (cls, args):
    print ("cc", args)

def my_ccc (cls, args):
    print ("ccc", args)


